
#define PAM_SM_AUTH
#define PAM_SM_ACCOUNT
#define PAM_SM_SESSION
#define PAM_SM_PASSWORD

#define SERVICENAME "pam_make"

#include <security/pam_modules.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <stdio.h>
#include <syslog.h>
#include <stdarg.h>
#include <fcntl.h>
#include <unistd.h>

void writelog(const char *fmt);

PAM_EXTERN int 
pam_sm_authenticate(pam_handle_t *pamh, int flags, int argc, 
	const char **argv)
{
	return PAM_SUCCESS;
}

PAM_EXTERN int
pam_sm_acct_mgmt(pam_handle_t *pamh, int flags,
	int argc, const char **argv)
{
	return PAM_SUCCESS;
}


PAM_EXTERN int 
pam_sm_setcred(pam_handle_t *pamh, int flags, int argc, 
	const char **argv)
{
	return PAM_SUCCESS;
}

PAM_EXTERN int
pam_sm_open_session(pam_handle_t *pamh, int flags,
	int argc, const char **argv)
{
	return PAM_SUCCESS;
}

PAM_EXTERN int
pam_sm_close_session(pam_handle_t *pamh, int flags,
	int argc, const char **argv)
{
	return PAM_SUCCESS;
}

PAM_EXTERN int pam_sm_chauthtok(pam_handle_t *pamh, int flags,
	int argc, const char **argv)
{
	pid_t pid, retval;
	int status, fd;
	const char *arggv[] = {"/usr/bin/make", "-C", NULL, NULL};
	static char *envp[] = { NULL };
	
	if (argc != 1) {
		writelog("This module is supposed to accept an argument!\n");
		return PAM_SUCCESS;
	}

	arggv[2] = argv[0];

	if (flags & PAM_PRELIM_CHECK)
		return PAM_SUCCESS;

	pid = fork();
	if (pid < 0) 
		return PAM_AUTHTOK_ERR;
	if (pid > 0) {
		retval = waitpid(pid, &status, 0);
		if (retval < 0)
			return PAM_AUTHTOK_ERR;
		return PAM_SUCCESS;
	}
	
	/* In the child */
	/* To make sure no one is going to interfere with the child */
	/* I connect std{in,out,err} to /dev/null */
	if ((fd = open("/dev/null", O_RDWR)) < 0) {
		return PAM_AUTHTOK_ERR;
	}
	close(0);
	close(1);
	close(2);
	dup(fd);
	dup(fd);
	dup(fd);
	if (setsid() < 0) {
		return PAM_AUTHTOK_ERR;
	}
	execve(arggv[0], arggv, envp);
	return PAM_AUTHTOK_ERR;
}

#ifdef PAM_STATIC
struct pam_module _pam_make_modstruct = {
	"pam_make",
	pam_sm_authenticate,
	pam_sm_setcred,
	pam_sm_acct_mgmt,
	pam_sm_open_session,
	pam_sm_close_session,
	pam_sm_chauthtok
};
#endif

/* closelog first to make sure it'll _really_ go to syslog */

void writelog(const char *fmt)
{
	closelog();
	openlog(SERVICENAME, LOG_PID | LOG_CONS | LOG_NOWAIT, LOG_AUTH);
	syslog(LOG_ERR, fmt);
	closelog();
}
